def {{problem_code}}(m, n, k, a):
    pass


if __name__ == "__main__":
    n = int(input())
    m, k = map(int, input().split())
    a = map(int, input().split())
    result = {{problem_code}}(m, n, k, a)
    print(result)

