# {{problem.name}}

{{problem.statement}}

## Input

{{problem.input_format}}

## Constraints

{{problem.constraints}}

## Output

{{problem.output_format}}

{% for testcase in problem.testcases_sample %}
## Sample input {{testcase.name}}

```
{{testcase.input}}
```

## Sample output {{testcase.name}}

```
{{testcase.output}}
```
{% if testcase.explanation %}
## Explanation {{testcase.name}}

{{testcase.explanation}}
{% endif %}
{% endfor %}