{
  "code": "{{problem.code}}",
  "slug": "{{problem.slug}}",
  "tags": {{tags}},
  "difficulty": {{problem.difficulty}},
  "experience_gain": {{problem.xp}},
  "statement_format": "{{problem.statement_format}}",
  "statement_language": "{{problem.statement_language}}",
  "statement_translations": {{statement_translations}},
  "limit_time_ms": {{problem.limit_time}},
  "limit_memory_mb": {{problem.limit_memory}},
  "testcase_format": "{{problem.testcase_format}}",
  "src_name": "{{problem.src_name}}",
  "src_id": "{{problem.src_id}}",
  "src_url": "{{problem.src_url}}"
}